@@ -2,12 +2,14 @@ extern crate askama;
 extern crate num;
 extern crate rand;
 extern crate serde;
+
 use askama::Template;
 use rand::Rng;
 use std::env;
 use std::fs::File;
 use std::io::Write;
 use std::process::Command;
+
 pub mod charts;
 
 use crate::charts::Layout;
@@ -21,6 +23,10 @@ struct PlotlyJs;
 struct PlotTemplate<'a> {
     plot_data: &'a str,
     plotly_javascript: &'a str,
+    export_image: bool,
+    image_type: &'a str,
+    image_width: usize,
+    image_height: usize,
 }
 
 pub trait TraceSerialize {
@@ -48,7 +54,13 @@ impl Plot {
         self.layout = Some(layout);
     }
 
-    fn render(&self) -> String {
+    fn render(
+        &self,
+        export_image: bool,
+        image_type: &str,
+        image_width: usize,
+        image_height: usize,
+    ) -> String {
         let mut plot_data = String::new();
         for (idx, trace) in self.traces.iter().enumerate() {
             let s = trace.serialize();
@@ -78,6 +90,10 @@ impl Plot {
         let tmpl = PlotTemplate {
             plot_data: plot_data.as_str(),
             plotly_javascript: plotly_js.as_str(),
+            export_image,
+            image_type,
+            image_width,
+            image_height,
         };
         tmpl.render().unwrap()
     }
@@ -97,14 +113,15 @@ impl Plot {
 
     #[cfg(target_os = "windows")]
     fn show_with_default_app(temp_path: &str) {
-        Command::new("start")
-            .args(&[r#""""#, temp_path])
+        Command::new("cmd")
+            .arg("/C")
+            .arg(format!(r#"start {}"#, temp_path))
             .output()
             .unwrap();
     }
 
     pub fn show(&self) {
-        let rendered = self.render();
+        let rendered = self.render(false, "", 0, 0);
         let rendered = rendered.as_bytes();
         let mut temp = env::temp_dir();
 
@@ -116,15 +133,64 @@ impl Plot {
 
         temp.push(plot_name);
         let temp_path = temp.to_str().unwrap();
-        let mut file = File::create(temp_path).unwrap();
-        file.write_all(rendered)
-            .expect("failed to write html output");
+        {
+            let mut file = File::create(temp_path).unwrap();
+            file.write_all(rendered)
+                .expect("failed to write html output");
+            file.flush();
+        }
 
         Plot::show_with_default_app(temp_path);
     }
 
-    pub fn to_file(&self, filename: &str) {
-        let rendered = self.render();
+    pub fn show_png(&self, width: usize, height: usize) {
+        let rendered = self.render(true, "png", width, height);
+        let rendered = rendered.as_bytes();
+        let mut temp = env::temp_dir();
+
+        let mut plot_name = rand::thread_rng()
+            .sample_iter(&rand::distributions::Alphanumeric)
+            .take(22)
+            .collect::<String>();
+        plot_name.push_str(".html");
+
+        temp.push(plot_name);
+        let temp_path = temp.to_str().unwrap();
+        {
+            let mut file = File::create(temp_path).unwrap();
+            file.write_all(rendered)
+                .expect("failed to write html output");
+            file.flush();
+        }
+
+        Plot::show_with_default_app(temp_path);
+    }
+
+    pub fn show_jpg(&self, width: usize, height: usize) {
+        let rendered = self.render(true, "jpg", width, height);
+        let rendered = rendered.as_bytes();
+        let mut temp = env::temp_dir();
+
+        let mut plot_name = rand::thread_rng()
+            .sample_iter(&rand::distributions::Alphanumeric)
+            .take(22)
+            .collect::<String>();
+        plot_name.push_str(".html");
+
+        temp.push(plot_name);
+        let temp_path = temp.to_str().unwrap();
+        {
+            let mut file = File::create(temp_path).unwrap();
+            file.write_all(rendered)
+                .expect("failed to write html output");
+            file.flush();
+        }
+
+        Plot::show_with_default_app(temp_path);
+    }
+
+    pub fn to_html(&self, filename: &str) {
+        let rendered = self.render(false, "", 0, 0);
         let rendered = rendered.as_bytes();
         let mut file = File::create(filename).unwrap();
         file.write_all(rendered)
